@@ -34,8 +34,8 @@ GPL.
 // enigmailCommon.js: shared JS functions for Enigmail
 
 // This Enigmail version and compatible Enigmime version
-var gEnigmailVersion = "0.95.3";
-var gEnigmimeVersion = "0.95.3.0";
+var gEnigmailVersion = "0.95.6";
+var gEnigmimeVersion = "0.95.6.0";
 
 // Maximum size of message directly processed by Enigmail
 const ENIG_MSG_BUFFER_SIZE = 96000;
@@ -50,6 +50,7 @@ const ENIG_PIPEFILTERLISTENER_CONTRACTID = "@mozilla.org/process/pipe-filter-lis
 const ENIG_ENIGMAIL_CONTRACTID    = "@mozdev.org/enigmail/enigmail;1";
 const ENIG_ENIGMIMELISTENER_CONTRACTID = "@mozilla.org/enigmail/mime-listener;1";
 const ENIG_ENIGMIMESERVICE_CONTRACTID = "@mozdev.org/enigmail/enigmimeservice;1";
+const ENIG_ENIGMIMEVERIFY_CONTRACTID = "@mozilla.org/enigmail/mime-verify;1";
 const ENIG_STRINGBUNDLE_CONTRACTID = "@mozilla.org/intl/stringbundle;1";
 const ENIG_LOCAL_FILE_CONTRACTID = "@mozilla.org/file/local;1";
 const ENIG_DIRSERVICE_CONTRACTID = "@mozilla.org/file/directory_service;1";
@@ -65,6 +66,7 @@ const ENIG_DATE_FORMAT_CONTRACTID = "@mozilla.org/intl/scriptabledateformat;1"
 const ENIG_ACCOUNT_MANAGER_CONTRACTID = "@mozilla.org/messenger/account-manager;1";
 const ENIG_XPCOM_APPINFO = "@mozilla.org/xre/app-info;1";
 
+
 const ENIG_LOCALFILEOUTPUTSTREAM_CONTRACTID =
                               "@mozilla.org/network/file-output-stream;1";
 const ENIG_STANDARD_URL_CONTRACTID = "@mozilla.org/network/standard-url;1";
@@ -1821,7 +1823,7 @@ function EnigCollapseAdvanced(obj, attribute, dummy) {
 
   var obj = obj.firstChild;
   while (obj) {
-    if (obj.getAttribute("advanced")) {
+    if (obj.getAttribute("advanced") == "true") {
       if (advancedUser) {
         obj.removeAttribute(attribute);
       }
@@ -1829,6 +1831,15 @@ function EnigCollapseAdvanced(obj, attribute, dummy) {
         obj.setAttribute(attribute, "true");
       }
     }
+    else if (obj.getAttribute("advanced") == "reverse") {
+      if (advancedUser) {
+        obj.setAttribute(attribute, "true");
+      }
+      else {
+        obj.removeAttribute(attribute);
+      }
+    }
+
     obj = obj.nextSibling;
   }
 }
\ No newline at end of file
