@@ -530,16 +530,11 @@ function enigMessageDecrypt(event, isAuto) {
       DEBUG_LOG("enigmailMessengerOverlay.js: "+headerName+": "+headerValue+"\n");
     }
 
-    // var emailAttachment = false;
     var embeddedSigned = null;
     var embeddedEncrypted = null;
     for (var indexb in currentAttachments) {
       var attachment = currentAttachments[indexb];
-/* obsolete
-      if (attachment.contentType.search(/^message\/rfc822(;|$)/i) == 0) {
-        emailAttachment = true;
-      }
-*/
+
       if (attachment.contentType.search(/^application\/pgp-signature/i) == 0) {
         embeddedSigned = attachment.url.replace(/\.\d+\.\d+$/, "");
       }
@@ -547,16 +542,8 @@ function enigMessageDecrypt(event, isAuto) {
         embeddedEncrypted = attachment.url.replace(/\.\d+\.\d+$/, "");
       }
       DEBUG_LOG("enigmailMessengerOverlay.js: "+indexb+": "+attachment.contentType+"\n");
-      //DEBUG_LOG("enigmailMessengerOverlay.js: "+indexb+": "+attachment.url+"\n");
     }
 
-/* Obsolete
-    if (emailAttachment) {
-      // DEBUG_LOG("enigmailMessengerOverlay.js: Email attachment; reloading to hide headers\n");
-      enigMessageReload(true);
-      return;
-    }
-*/
   }
 
   var contentType = "";
@@ -615,6 +602,14 @@ function enigMessageDecrypt(event, isAuto) {
     else if (! isAuto) {
       enigMessageReload(false);
     }
+    else if (embeddedEncrypted && (! encrypedMsg)) {
+      var mailNewsUrl = enigGetCurrentMsgUrl();
+      if (mailNewsUrl) {
+        mailNewsUrl.spec = embeddedEncrypted;
+        enigVerifyEmbeddedMsg(window, mailNewsUrl, msgWindow, msgUriSpec, contentEncoding, event);
+      }
+    }
+
     return;
   }
 
@@ -649,17 +644,17 @@ function enigMessageDecrypt(event, isAuto) {
       return;
 
     var mailNewsUrl = enigGetCurrentMsgUrl();
-    if (embeddedSigned) {
-      mailNewsUrl.spec = embeddedSigned;
-    }
-
     if (mailNewsUrl) {
-      const ENIG_ENIGMIMEVERIFY_CONTRACTID = "@mozilla.org/enigmail/mime-verify;1";
-      var verifier = Components.classes[ENIG_ENIGMIMEVERIFY_CONTRACTID].createInstance(Components.interfaces.nsIEnigMimeVerify);
-
-      verifier.init(window, mailNewsUrl, msgWindow, msgUriSpec,
-                    true, enableSubpartTreatment);
+      if (embeddedSigned) {
+        mailNewsUrl.spec = embeddedSigned;
+        enigVerifyEmbeddedMsg(window, mailNewsUrl, msgWindow, msgUriSpec, contentEncoding, event);
+      }
+      else {
+        var verifier = Components.classes[ENIG_ENIGMIMEVERIFY_CONTRACTID].createInstance(Components.interfaces.nsIEnigMimeVerify);
 
+        verifier.init(window, mailNewsUrl, msgWindow, msgUriSpec,
+                      true, enableSubpartTreatment);
+      }
       return;
     }
   }
@@ -679,7 +674,10 @@ function enigMessageParse(interactive, importOnly, contentEncoding) {
   DEBUG_LOG("enigmailMessengerOverlay.js: bodyElement="+bodyElement+"\n");
 
   var findStr = interactive ? null : "-----BEGIN PGP";
-  var msgText = EnigGetDeepText(bodyElement, findStr);
+  var msgText = null;
+
+  if ((! findStr) || (findStr && (bodyElement.textContent.indexOf(findStr) >= 0)))
+    msgText = bodyElement.textContent;
 
   if (!msgText) {
     // No PGP content
@@ -691,6 +689,16 @@ function enigMessageParse(interactive, importOnly, contentEncoding) {
   // Encode ciphertext to charset from unicode
   msgText = EnigConvertFromUnicode(msgText, charset);
 
+  var mozPlainText = bodyElement.innerHTML.search(/class=\"moz-text-plain\"/);
+  if ((mozPlainText >= 0) && (mozPlainText < 40)) {
+    // workaround for too much expanded emoticons in plaintext msg
+    var r = new RegExp(/( )(;-\)|:-\)|;\)|:\)|:-\(|:\(|:-\\|:-P|:-D|:-\[|:-\*|\>:o|8-\)|:-\$|:-X|\=-O|:-\!|O:-\)|:\'\()( )/g);
+    if (msgText.search(r) >= 0) {
+      DEBUG_LOG("enigmailMessengerOverlay.js: enigMessageParse: performing emoticons fixing\n");
+      msgText = msgText.replace(r, "$2");
+    }
+  }
+
   // extract text preceeding and/or following armored block
   var head="";
   var tail="";
@@ -710,8 +718,10 @@ function enigMessageParse(interactive, importOnly, contentEncoding) {
 
   var urlSpec = mailNewsUrl ? mailNewsUrl.spec : "";
 
+  retry = (charset != "UTF-8" ? 1 : 2);
+
   enigMessageParseCallback(msgText, contentEncoding, charset, interactive,
-                           importOnly, urlSpec, "", true, head, tail);
+                           importOnly, urlSpec, "", retry, head, tail);
 }
 
 
@@ -824,12 +834,25 @@ function enigMessageParseCallback(msgText, contentEncoding, charset, interactive
 
   if ((exitCode !=0) && (! (statusFlags & noSecondTry))) {
     // Bad signature/armor
-    if (retry) {
+    if (retry == 1) {
+      msgText = EnigConvertFromUnicode(msgText, "UTF-8");
+      enigMessageParseCallback(msgText, contentEncoding, charset, interactive,
+                               importOnly, messageUrl, signature, retry + 1,
+                               head, tail)
+    }
+    else if (retry == 2) {
       // Try to verify signature by accessing raw message text directly
       // (avoid recursion by setting retry parameter to false on callback)
+      newSignature  = "";
       enigMsgDirect(interactive, importOnly, contentEncoding, charset, newSignature, 0, head, tail, enigMessageParseCallback);
       return;
     }
+    else if (retry == 3) {
+      msgText = EnigConvertToUnicode(msgText, "UTF-8");
+      enigMessageParseCallback(msgText, contentEncoding, charset, interactive,
+                               importOnly, messageUrl, null, retry + 1,
+                               head, tail)
+    }
   }
 
   if (!plainText) {
@@ -1229,22 +1252,29 @@ function enigMsgPrint(elementId) {
 
   var contextMenu = (elementId.search("Context") > -1);
 
-  if (!gEnigDecryptedMessage)
+  if (!gEnigDecryptedMessage || typeof(gEnigDecryptedMessage) == "undefined") {
     enigMsgDefaultPrint(elementId);
+    return;
+  }
 
   var mailNewsUrl = enigGetCurrentMsgUrl();
 
-  if (!mailNewsUrl)
+  if (!mailNewsUrl) {
     enigMsgDefaultPrint(elementId);
+    return
+  }
 
   if (gEnigDecryptedMessage.url != mailNewsUrl.spec) {
     gEnigDecryptedMessage = null;
     enigMsgDefaultPrint(elementId);
+    return;
   }
 
   var enigmailSvc = GetEnigmailSvc();
-  if (!enigmailSvc)
+  if (!enigmailSvc) {
     enigMsgDefaultPrint(elementId);
+    return;
+  }
 
   // Note: Trying to print text/html content does not seem to work with
   //       non-ASCII chars
@@ -1328,6 +1358,7 @@ function enigMsgDirect(interactive, importOnly, contentEncoding, charset, signat
     return;
 
   var ipcBuffer = Components.classes[ENIG_IPCBUFFER_CONTRACTID].createInstance(Components.interfaces.nsIIPCBuffer);
+  var mimeListener = Components.classes[ENIG_ENIGMIMELISTENER_CONTRACTID].createInstance(Components.interfaces.nsIEnigMimeListener);
 
   if (bufferSize > 0) {
     ipcBuffer.open(bufferSize, false);
@@ -1346,6 +1377,7 @@ function enigMsgDirect(interactive, importOnly, contentEncoding, charset, signat
                       expectedBufferSize: bufferSize,
                       head:head,
                       tail:tail,
+                      mimeListener: mimeListener,
                       callbackFunction: callbackFunction };
 
   var requestObserver = new EnigRequestObserver(enigMsgDirectCallback,
@@ -1358,7 +1390,6 @@ function enigMsgDirect(interactive, importOnly, contentEncoding, charset, signat
   var channel = ioServ.newChannelFromURI(mailNewsUrl);
 
   var pipeFilter = Components.classes[ENIG_PIPEFILTERLISTENER_CONTRACTID].createInstance(Components.interfaces.nsIPipeFilterListener);
-
   pipeFilter.init(ipcBuffer, null,
                 "-----BEGIN PGP",
                 "-----END PGP",
@@ -1367,7 +1398,6 @@ function enigMsgDirect(interactive, importOnly, contentEncoding, charset, signat
   var listener;
 
   try {
-    var mimeListener = Components.classes[ENIG_ENIGMIMELISTENER_CONTRACTID].createInstance(Components.interfaces.nsIEnigMimeListener);
 
     mimeListener.init(pipeFilter, null, ENIG_MSG_HEADER_SIZE, true, false, true);
 
@@ -1377,7 +1407,7 @@ function enigMsgDirect(interactive, importOnly, contentEncoding, charset, signat
     listener = pipeFilter;
   }
 
-  channel.asyncOpen(listener, mailNewsUrl);
+  channel.asyncOpen(pipeFilter, mailNewsUrl);
 }
 
 
@@ -1388,7 +1418,9 @@ function enigMsgDirectCallback(callbackArg, ctxt) {
   var urlSpec = mailNewsUrl ? mailNewsUrl.spec : "";
   var newBufferSize = 0;
 
-  if (urlSpec != callbackArg.messageUrl) {
+  var l= urlSpec.length;
+
+  if (urlSpec.substr(0, l) != callbackArg.messageUrl.substr(0, l)) {
     ERROR_LOG("enigmailMessengerOverlay.js: enigMsgDirectCallback: Message URL mismatch "+mailNewsUrl.spec+" vs. "+callbackArg.messageUrl+"\n");
     return;
   }
@@ -1402,6 +1434,7 @@ function enigMsgDirectCallback(callbackArg, ctxt) {
   }
 
   var msgText = callbackArg.ipcBuffer.getData();
+  msgText = EnigConvertFromUnicode(msgText, "UTF-8");
 
   callbackArg.ipcBuffer.shutdown();
 
@@ -1426,12 +1459,70 @@ function enigMsgDirectCallback(callbackArg, ctxt) {
                            callbackArg.importOnly,
                            callbackArg.messageUrl,
                            callbackArg.signature,
-                           false,
+                           3,
                            callbackArg.head,
                            callbackArg.tail);
 }
 
 
+function enigVerifyEmbeddedMsg(window, msgUrl, msgWindow, msgUriSpec, contentEncoding, event) {
+  DEBUG_LOG("enigmailMessengerOverlay.js: enigVerifyEmbedded: msgUrl"+msgUrl+"\n");
+
+  var ipcBuffer = Components.classes[ENIG_IPCBUFFER_CONTRACTID].createInstance(Components.interfaces.nsIIPCBuffer);
+  ipcBuffer.open(ENIG_MSG_BUFFER_SIZE, false);
+
+  var callbackArg = { ipcBuffer: ipcBuffer,
+                      window: window,
+                      msgUrl: msgUrl,
+                      msgWindow: msgWindow,
+                      msgUriSpec: msgUriSpec,
+                      contentEncoding: contentEncoding,
+                      event: event };
+
+  var requestObserver = new EnigRequestObserver(enigVerifyEmbeddedCallback,
+                                                callbackArg);
+
+  ipcBuffer.observe(requestObserver, msgUrl);
+
+  var ioServ = Components.classes[ENIG_IOSERVICE_CONTRACTID].getService(Components.interfaces.nsIIOService);
+
+  var channel = ioServ.newChannelFromURI(msgUrl);
+
+
+  var pipeFilter = Components.classes[ENIG_PIPEFILTERLISTENER_CONTRACTID].createInstance(Components.interfaces.nsIPipeFilterListener);
+
+  pipeFilter.init(ipcBuffer, null,
+                "",
+                "",
+                0, false, false, null);
+  channel.asyncOpen(pipeFilter, msgUrl);
+}
+
+
+function enigVerifyEmbeddedCallback(callbackArg, ctxt) {
+  DEBUG_LOG("enigmailMessengerOverlay.js: enigVerifyEmbeddedCallback: "+ctxt+"\n");
+
+  var txt = callbackArg.ipcBuffer.getData();
+  callbackArg.ipcBuffer.shutdown();
+
+  if (txt.length > 0 && txt.search(/content\-type:[ \t]*multipart\/signed/i) >= 0) {
+    // Real multipart/signed message; let's try to verify it
+    DEBUG_LOG("enigmailMessengerOverlay.js: enigVerifyEmbeddedCallback: detected multipart/signed\n");
+
+    var verifier = Components.classes[ENIG_ENIGMIMEVERIFY_CONTRACTID].createInstance(Components.interfaces.nsIEnigMimeVerify);
+
+    verifier.init(callbackArg.window, callbackArg.msgUrl, callbackArg.msgWindow, callbackArg.msgUriSpec,
+                      true, true);
+  }
+  else {
+    // try inline PGP
+    DEBUG_LOG("enigmailMessengerOverlay.js: enigVerifyEmbeddedCallback: try inline PGP\n");
+
+    enigMessageParse(!callbackArg.event, false, callbackArg.contentEncoding);
+  }
+}
+
+
 function enigKeyRequest(interactive, keyId, urlSpec) {
   DEBUG_LOG("enigmailMessengerOverlay.js: enigKeyRequest: keyId="+keyId+", urlSpec="+urlSpec+"\n");
 
