@@ -74,10 +74,8 @@ if (typeof(GenericSendMessage)=="function") {
   window.addEventListener('compose-window-close', enigMsgComposeClose, true);
   window.addEventListener('compose-window-reopen', enigMsgComposeReopen, true);
 
-  /*
-    // Handle message sending (in future)
-    window.addEventListener('compose-send-message', enigMsgComposeSendMsg, true);
-  */
+  // Listen to message sending event
+  window.addEventListener('compose-send-message', enigSendMessageListener, true);
 }
 else {
   EnigAlert("WARNING:\n\nCannot hook message sending -- you will not be able to send OpenPGP messages!");
@@ -708,20 +706,13 @@ function enigEncryptMsg(msgSendType) {
   var sendFlags=0;
   window.enigmailSendFlags=0;
 
-  var autoSaveAsDraft = nsIMsgCompDeliverMode.SaveAsDraft;
-  try {
-    // TB 1.5 only
-    autoSaveAsDraft = nsIMsgCompDeliverMode.AutoSaveAsDraft;
-  }
-  catch (ex) {}
-
   switch (msgSendType) {
   case nsIMsgCompDeliverMode.Later:
     sendFlags |= nsIEnigmail.SEND_LATER;
     break;
   case nsIMsgCompDeliverMode.SaveAsDraft:
   case nsIMsgCompDeliverMode.SaveAsTemplate:
-  case autoSaveAsDraft:
+  case nsIMsgCompDeliverMode.AutoSaveAsDraft:
     sendFlags |= nsIEnigmail.SAVE_MESSAGE;
     break;
   }
@@ -736,8 +727,7 @@ function enigEncryptMsg(msgSendType) {
       newsgroups == "") {
     // don't attempt to send message if no recipient specified
     EnigAlert(sComposeMsgsBundle.getString("12511"));
-    window.cancelSendMessage=true;
-    return;
+    return false;
   }
 
   if (gotSendFlags & ENIG_SIGN)
@@ -749,14 +739,13 @@ function enigEncryptMsg(msgSendType) {
   if (sendFlags & nsIEnigmail.SAVE_MESSAGE) {
     if (!((sendFlags & ENIG_ENCRYPT) && EnigConfirmPref(EnigGetString("savingMessage"), "saveEncrypted"))) {
       sendFlags &= ~ENIG_ENCRYPT;
-      return;
+      return true;
     }
   }
 
   if (gWindowLocked) {
     EnigAlert(EnigGetString("windowLocked"));
-    window.cancelSendMessage=true;
-    return;
+    return false;
   }
 
   if (gEnigDirty) {
@@ -764,12 +753,10 @@ function enigEncryptMsg(msgSendType) {
     // (it may have been set by a previously cancelled send operation!)
     try {
       if (gMsgCompose.compFields.securityInfo instanceof Components.interfaces.nsIEnigMsgCompFields) {
-        if (!gEnigSendPGPMime) {
-          gMsgCompose.compFields.securityInfo=null;
-        }
-        else {
-          gMsgCompose.compFields.securityInfo.sendFlags=0;
-        }
+        gMsgCompose.compFields.securityInfo.sendFlags=0;
+      }
+      else if (gMsgCompose.compFields.securityInfo == null) {
+        throw "dummy";
       }
     }
     catch (ex){
@@ -794,9 +781,7 @@ function enigEncryptMsg(msgSendType) {
         msg = gEnigmailSvc.initializationError +"\n\n"+msg;
      }
 
-     if (!EnigConfirm(msg))
-        window.cancelSendMessage=true;
-     return;
+     return EnigConfirm(msg);
   }
 
 
@@ -815,9 +800,9 @@ function enigEncryptMsg(msgSendType) {
      if (! pgpEnabled) {
         if (sendFlags & ENIG_ENCRYPT_OR_SIGN) {
           if (!EnigConfirm(EnigGetString("acctNotConfigured")))
-              window.cancelSendMessage=true;
+              return false;
         }
-        return;
+        return true;
      }
 
      var recipientsSelection = EnigGetPref("recipientsSelection");
@@ -861,18 +846,18 @@ function enigEncryptMsg(msgSendType) {
 
      if (typeof(userIdValue) != "string") {
        DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: type of userIdValue="+typeof(userIdValue)+"\n");
-       userIdValue = "";
+       userIdValue = gEnigIdentity.email;
      }
 
      DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg:gMsgCompose="+gMsgCompose+"\n");
 
      var toAddrList = [];
      if (sendFlags & nsIEnigmail.SAVE_MESSAGE) {
-        if (userIdValue.search(/@/) == -1) {
+        if (userIdValue.search(/@/) == -1 ) {
           toAddrList.push(userIdValue);
         }
         else {
-          enigAddRecipients(toAddrList, [ userIdValue ]);
+          toAddrList.push(EnigStripEmail(userIdValue.replace(/[\",]/g, "")));
         }
      }
      else {
@@ -907,8 +892,7 @@ function enigEncryptMsg(msgSendType) {
 
            } else {
              if (!EnigConfirm(EnigGetString("sendingBCC"))) {
-               window.cancelSendMessage=true;
-               return;
+               return false;
              }
            }
          }
@@ -921,8 +905,7 @@ function enigEncryptMsg(msgSendType) {
 
            if (!encryptIfPossible) {
              EnigAlert(EnigGetString("sendingNews"));
-             window.cancelSendMessage=true;
-             return;
+             return false;
            }
 
            sendFlags &= ~ENIG_ENCRYPT;
@@ -957,8 +940,7 @@ function enigEncryptMsg(msgSendType) {
                                   (repeatSelection==1),
                                   matchedKeysObj,
                                   flagsObj)) {
-              window.cancelSendMessage=true;
-              return;
+              return false;
             }
 
             if (matchedKeysObj.value) toAddr=matchedKeysObj.value;
@@ -1032,8 +1014,7 @@ function enigEncryptMsg(msgSendType) {
                 window.openDialog("chrome://enigmail/content/enigmailUserSelection.xul","", "dialog,modal,centerscreen", inputObj, resultObj);
                 try {
                   if (resultObj.cancelled) {
-                    window.cancelSendMessage=true;
-                    return;
+                    return false;
                   }
                   if (resultObj.perRecipientRules && gEnigEnableRules) {
                     // do an extra round because the user want to set a PGP rule
@@ -1050,8 +1031,7 @@ function enigEncryptMsg(msgSendType) {
                   testExitCodeObj.value = 0;
                 } catch (ex) {
                   // cancel pressed -> don't send mail
-                  window.cancelSendMessage=true;
-                  return;
+                  return false;
                 }
             }
             if ((!testCipher || (testExitCodeObj.value != 0)) && recipientsSelection==5) {
@@ -1121,8 +1101,7 @@ function enigEncryptMsg(msgSendType) {
           }
           if (resultObj.selected < 0) {
             // dialog cancelled
-            window.cancelSendMessage=true;
-            return;
+            return false;
           }
           else if (resultObj.selected == 1) {
             // encrypt attachments
@@ -1136,8 +1115,7 @@ function enigEncryptMsg(msgSendType) {
         else {
           if (sendFlags & ENIG_ENCRYPT) {
             if (!EnigConfirm(EnigGetString("attachWarning")))
-              window.cancelSendMessage=true;
-              return;
+              return false;
           }
         }
      }
@@ -1160,8 +1138,7 @@ function enigEncryptMsg(msgSendType) {
             if (gMsgCompose.compFields.securityInfo.requireEncryptMessage ||
               gMsgCompose.compFields.securityInfo.signMessage) {
                 EnigAlert("pgpMime.sMime.incomaptible");
-                window.cancelSendMessage=true;
-                return;
+                return false;
             }
         }
      }
@@ -1217,8 +1194,7 @@ function enigEncryptMsg(msgSendType) {
        };
 
        if (! enigEncryptInline(sendInfo)) {
-         window.cancelSendMessage=true;
-         return;
+         return false;
        }
      }
 
@@ -1234,8 +1210,7 @@ function enigEncryptMsg(msgSendType) {
          if (gEnigProcessed)
            enigUndoEncryption();
 
-         window.cancelSendMessage=true;
-         return;
+         return false;
        }
      }
      else if ( (sendFlags & nsIEnigmail.SEND_WITH_CHECK) &&
@@ -1244,19 +1219,27 @@ function enigEncryptMsg(msgSendType) {
        if (gEnigProcessed)
          enigUndoEncryption();
 
-       window.cancelSendMessage=true;
-       return;
+       return false;
      }
 
-     if (usingPGPMime &&
-         ((sendFlags & ENIG_ENCRYPT_OR_SIGN))) {
-      // temporarily enable quoted-printable for PGP/MIME messages
+
+     var warnIso2022jp="Your message is encoded with the Japanese character set ISO-2022-JP. Unfortunately, due to technical reasons comaptibility to the OpenPGP standards cannot be ensured with this character set. Do you want to convert the message to UTF-8 instead?";
+
+
+     if (usingPGPMime && ((sendFlags & ENIG_ENCRYPT_OR_SIGN))) {
+       // temporarily enable quoted-printable for PGP/MIME messages
        try {
           // make sure plaintext is not changed to 7bit
           if (typeof(msgCompFields.forceMsgEncoding) == "boolean") {
             msgCompFields.forceMsgEncoding = true;
             DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: enabled forceMsgEncoding\n");
           }
+
+          if (msgCompFields.characterSet == "ISO-2022-JP") {
+            if (EnigConfirmPref(warnIso2022jp, "warnIso2022jp")) {
+              gMsgCompose.SetDocumentCharset("UTF-8");
+            }
+          }
        }
        catch (ex) {}
     }
@@ -1265,6 +1248,12 @@ function enigEncryptMsg(msgSendType) {
         // force keeping the charset (i.e. don't convert to us-ascii)
         msgCompFields.forceMsgEncoding = true;
         DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: enabled forceMsgEncoding\n");
+
+        if (msgCompFields.characterSet == "ISO-2022-JP") {
+          if (EnigConfirmPref(warnIso2022jp, "warnIso2022jp")) {
+            gMsgCompose.SetDocumentCharset("UTF-8");
+          }
+        }
       }
     }
 
@@ -1274,9 +1263,10 @@ function enigEncryptMsg(msgSendType) {
      if (gEnigmailSvc && gEnigmailSvc.initializationError) {
         msg += "\n"+gEnigmailSvc.initializationError;
      }
-     if (!EnigConfirm(msg))
-       window.cancelSendMessage=true;
+     return EnigConfirm(msg);
   }
+
+  return true;
 }
 
 function enigEncryptInline(sendInfo) {
@@ -1494,7 +1484,6 @@ function enigMessageSendCheck() {
 /////////////////////////////////////////////////////////////////////////
 // Call the following function from our version of the function
 // GenericSendMessage from the file MsgComposeCommands.js
-// (after the calls to Recipients2CompFields and Attachements2CompFields)
 /////////////////////////////////////////////////////////////////////////
 function enigModifyCompFields(msgCompFields) {
 
@@ -1529,15 +1518,29 @@ function enigModifyCompFields(msgCompFields) {
     }
   }
   catch (ex) {
-    EnigWriteException("enigmailMsgComposeOverlay.js: enigModifyCompFields", ex)
+    EnigWriteException("enigmailMsgComposeOverlay.js: enigModifyCompFields", ex);
   }
 
   DEBUG_LOG("enigmailMsgComposeOverlay.js: enigModifyCompFields: otherRandomHeaders = "+
            msgCompFields.otherRandomHeaders+"\n");
 }
 
+function enigSendMessageListener(event) {
+  DEBUG_LOG("enigmailMsgComposeOverlay.js: enigSendMessageListener\n");
+  try {
+    var msgcomposeWindow = document.getElementById("msgcomposeWindow");
+    enigModifyCompFields(gMsgCompose.compFields);
+    if (! enigEncryptMsg(Number(msgcomposeWindow.getAttribute("msgtype")))) {
+      event.preventDefault();
+      event.stopPropagation();
+    }
+  }
+  catch (ex) {}
+}
+
+// GenericSendMessage from the file MsgComposeCommands.js (backported from TB 3.0)
+// only used for TB 2.0
 
-// Modified version of GenericSendMessage from the file MsgComposeCommands.js
 function enigGenericSendMessage( msgType )
 {
   DEBUG_LOG("enigmailMsgComposeOverlay.js: enigGenericSendMessage: msgType="+msgType+"\n");
@@ -1721,32 +1724,18 @@ function enigGenericSendMessage( msgType )
           gMsgCompose.SetDocumentCharset(fallbackCharset);
       }
 
-
-/////////////////////////////////////////////////////////////////////////
-// MODIFICATION
-// Call the following function from our version of the function
-// GenericSendMessage from the file MsgComposeCommands.js
-// (after having determined HTML convertibility)
-/////////////////////////////////////////////////////////////////////////
-      try {
-        window.cancelSendMessage=false;
-        enigModifyCompFields(msgCompFields);
-        enigEncryptMsg(msgType);
-        if(window.cancelSendMessage)
-          return;
-      }
-      catch(ex){}
-/////////////////////////////////////////////////////////////////////////
-// END OF MODIFICATION
-/////////////////////////////////////////////////////////////////////////
-
       try {
 
         // just before we try to send the message, fire off the compose-send-message event for listeners
         // such as smime so they can do any pre-security work such as fetching certificates before sending
-        var event = document.createEvent('Events');
+        var event = document.createEvent('UIEvents');
         event.initEvent('compose-send-message', false, true);
-        document.getElementById("msgcomposeWindow").dispatchEvent(event);
+        var msgcomposeWindow = document.getElementById("msgcomposeWindow");
+        msgcomposeWindow.setAttribute("msgtype", msgType);
+        msgcomposeWindow.dispatchEvent(event);
+        if (event.getPreventDefault())
+          throw Components.results.NS_ERROR_ABORT;
+
         gAutoSaving = (msgType == nsIMsgCompDeliverMode.AutoSaveAsDraft);
         // disable the ui if we're not auto-saving
         if (!gAutoSaving)
@@ -1777,20 +1766,6 @@ function enigGenericSendMessage( msgType )
         }
         msgWindow.rootDocShell.allowAuth = true;
         gMsgCompose.SendMsg(msgType, getCurrentIdentity(), currentAccountKey, msgWindow, progress);
-/*
-/////////////////////////////////////////////////////////////////////////
-// MODIFICATION
-// Call the following function from our version of the function
-// GenericSendMessage from the file MsgComposeCommands.js
-// (after gMsgCompose.SendMsg)
-/////////////////////////////////////////////////////////////////////////
-        if (window.enigmailSendFlags & nsIEnigmail.SAVE_MESSAGE) {
-          enigUndoEncryption();
-        }
-/////////////////////////////////////////////////////////////////////////
-// END OF MODIFICATION
-/////////////////////////////////////////////////////////////////////////
-*/
       }
       catch (ex) {
         dump("failed to SendMsg: " + ex + "\n");
